#!/usr/bin/python3

#TODO catch if no ola is installed
# Traceback (most recent call last):
#   File "./skyglow.py", line 5, in <module>
#     from ola.ClientWrapper import ClientWrapper
# ImportError: No module named ola.ClientWrapper


from __future__ import print_function #python 2
import os
from ola.ClientWrapper import ClientWrapper
from datetime import datetime as dt
import array
import sys

# error / return codes
# # # # # # #
FAIL_DMX=3
FAIL_DMX_SEND=4
FAIL_DMX_RECEIVE=5
FAIL_LOG=6
FAIL_MOON=7

# either fullpath or just name (then logFile will be created in scriptdir)
# or fullpath containing '/'
logFile = "skyglow.log"
data = []
moonLux=0
moonDmx=0

#dmx setup
wrapper = None
UNIVERSE = 10
dmxDataReceived = False

MAX_DMX_FRAME_SIZE = 512

#change according to needs
dmx_addr = [10,20,30,40,50,60,70,80,90,100,110,120]

# assert len(dmx_addr) == NUM_ECO_UNITS 
# assert len(skyglow) == NUM_ECO_UNITS

#Mapping byte to lux
skyglow = [ 15, # one 0.08
			54, # one 0.3
			 4, # all 1
			19,	# one 0.1
			 0,	# none 0
			 1,	# one 0.01
			 5,	# one 0.03
			 0,	# none 0
			40,	# all 10
			12,	# all 3
		   123,	# all 30
		   123,	# all 30
]

NUM_ECO_UNITS = len(skyglow)
sunrise = 0
sunset = 0
#datetime.now(timezone.utc) #create aware!
#utcnow() / utcfromtimestamp()
now = dt.now().replace(second=0, microsecond=0) # ignore seconds and microseconds

#mlx (idx is also dmx-byte value)
moon = [ 1.409,
		 8.369,
		12.333,
		16.210,
		20.563,
		24.716,
		28.907,
		33.363,
		36.843,
		41.137,
		46.527,
		50.880,
		54.727,
		59.617,
		63.967,
		67.817,
		73.173,
		77.213,
		82.827,
		87.037,
		91.033,
		95.320,
		99.470,
		104.366,
		108.833,
		113.667, #25
		123.733,
		128.567,
		133.2,
		138.3,
		143.367,
		148.233,
		152.233,
		156.933,
		162.5,
		167.5,
		172.1,
		176.767,
		181.567,
		187,
		192.067,
		196.433,
		201.467,
		206.5,
		211.733,
		216.4,
		221.033,
		226,
		231.1,
		235.733,
		240.4, #50
		245.6,
		250.433,
		254.567,
		259.5,
		263.933,
		268.3,
		272.6, #274 == new max
		277.267,
		281.433,
		287.467, #60
	]
	

#moonFile
DATE_COLUMN=0
TIME_COLUMN=1
MOON_COLUMN=13
IGNORE_FIRST_LINES=5

def moonlux2Byte(m):
	'''find minimum Byte/Index in moonarray'''

	m=m*1000
	min_i = 0
	min_m = max(moon)
	min_dif = 999
	
	for i in range(len(moon)):
		if abs(m - moon[i]) < min_dif:
				min_dif = abs(m - moon[i])
				min_i = i

	return min_i


def readLux():
	''' read current lux-value file and transfer to byte-value '''

	if not os.path.isfile(moonFile) or not os.access(moonFile, os.R_OK):
		error(FAIL_MOON,"moonfile not accessable: " + moonFile)

	with open(moonFile) as f:
		lines = f.readlines()

	del lines[0:IGNORE_FIRST_LINES] #remove no data lines

	moonLux = float()
	for line in lines:

		line=line.split()
		if len(line) < MOON_COLUMN+1:
			continue

		sdatetime = line[DATE_COLUMN] + " " + line[TIME_COLUMN]
		
		parsed_datetime = dt.strptime(sdatetime,'%m-%d-%Y %H:%M')
		# print(parsed_datetime)

		if parsed_datetime == now:
			moonLux = line[MOON_COLUMN]
			print(str(now) + ": got moon illuminance of " + moonLux)
			break;

	return moonLux

def log(msg):
	msg = str(now) + msg + "\n"
	
	if not os.access(logFile, os.X_OK) or os.access(logFile, os.W_OK):
		print("writing log to file ", logFile)
		with open(logFile, 'a') as f:
			f.write(msg)
			print(msg)
	else:
		print("Error! File: Can't write log to ",logFile)
		exit(FAIL_LOG)

def error(exit_code, msg):
	log("Error! " + str(msg))
	exit(exit_code)

def addPathTo(filename):
	cwd = os.path.realpath(__file__)  # cwd = os.getcwd()	
	cwd = cwd[0:cwd.rindex('/')]
	if '/' not in filename:
		filename = cwd + "/" + filename
	return filename 

def init():
	global logFile, moonFile, data
	global sunrise, sunset, now

	moonFile = 'moon'
	if len(sys.argv) >= 2:
		moonFile = sys.argv[1]
	
	moonFile= addPathTo(moonFile)
	logFile = addPathTo(logFile)

	CHANNELS_PER_UNIT = 10 #4 used. 6reserved
	USED_CHANNELS_PER_UNIT = 4
	# data = array.array('B',[0]*130)
	data = array.array('B',[0]*(((NUM_ECO_UNITS+1)*CHANNELS_PER_UNIT)-(NUM_ECO_UNITS*USED_CHANNELS_PER_UNIT))) # B=unsigned int

	# timestring="09-09-2020 18:00"
	# now = dt.strptime(timestring,'%m-%d-%Y %H:%M')

	if( (now.month == 7) or (now.month == 8 and now.day <= 18 )):
		sunrise = 6
		sunset = 21
	elif( now.month == 8 or (now.month == 9 and now.day <= 15 )):
		sunrise = 7
		sunset = 20
	else:
		sunrise = 7
		sunset = 19

def isDay():
	return ((now.hour >= sunrise) and (now.hour < sunset))

def dmxSent(status):
	if not status.Succeeded():
		error(FAIL_DMX_SEND,status.message)

	global wrapper
	if wrapper:
		wrapper.Stop()

	# only log after dmx status rcv
	if isDay():
		log('\t' + str(moonLux) + "\t" + str(moonDmx) + "\t[" + "0, "*(len(skyglow)-1) + "0]")
	else:
		log('\t' + str(moonLux) + "\t" + str(moonDmx) + "\t" + str(skyglow))

def dmxReceive(data):
  print("Dmx, data received:" + data)

def main(args=""):

	print("welcome to skyglow!")
	print(args)

	global data, moonLux, moonDmx
	init()
	
	moonLux = float(readLux())
	moonDmx = moonlux2Byte(moonLux)
	# print("moonDmx: ", moonDmx, "moonLux: ", moonLux)
	
	for eco_unit in range(NUM_ECO_UNITS):
		base = dmx_addr[eco_unit]

		# print(now, sunrise, sunset)
		
		if isDay():
			pollution = 0
		else:
			pollution = skyglow[eco_unit]

		print("unit:", eco_unit+1, "  dmxaddr:", base,"  moon:", moonDmx, "  pollution:", pollution)
		
		if eco_unit in [2,8,9,10,11]:
			data.insert(base-1, pollution)
			data.insert(base, 	pollution)
			data.insert(base+1, moonDmx)
			data.insert(base+2, pollution)
		else:
			data.insert(base-1, pollution)
			data.insert(base, 	0)
			data.insert(base+1, moonDmx)
			data.insert(base+2, 0)
	
	while( len(data) > MAX_DMX_FRAME_SIZE ):
		print("Warning: cutting off data to fit DMX frame size of ", MAX_DMX_FRAME_SIZE)
		data.pop()

	print("DMX-Data:",data)

	try:
		global wrapper
		wrapper = ClientWrapper()
		client = wrapper.Client()
		# send 1 dmx frame with values for channels 1-3 for all ecounits
		client.SendDmx(UNIVERSE, data, dmxSent)
	  	client.RegisterUniverse(UNIVERSE, client.REGISTER, dmxReceive)
		wrapper.Run()
	except Exception as e:
		error(FAIL_DMX,e)

	if not dmxDataReceived:
		error(FAIL_DMX_RECEIVE)


if __name__ == '__main__':
	main()

